package edu.gac.max.puzzle;

/**
 * @author Max Hailperin, Barbara Kaiser, and Karl Knight
 * 
 * This file contains excerpts from the textbook Concrete Abstractions: An Introduction to Computer Science Using Scheme,
 * by Max Hailperin, Barbara Kaiser, and Karl Knight, Copyright (c) 1998 by the authors. Full text is available for free
 * at http://www.gustavus.edu/+max/concrete-abstractions.html
 * 
 * Modified by Max Hailperin in November 2013 to separate model from view.
 */
public class LocalModel implements Model {
	
	private String[][] tiles;
	private int blankRow;
	private int blankCol;

	public LocalModel(){
		tiles = new String[Puzzle.SIZE][Puzzle.SIZE];
		initializeTiles();
	}
	
	public void initializeTiles(){
		int tileCount = 0;
		for(int row = 0; row < Puzzle.SIZE; row++){
			for(int col = 0; col < Puzzle.SIZE; col++){
				tileCount++;
				tiles[row][col] = "" + tileCount;
			}
		}
		blankRow = Puzzle.SIZE - 1;
		blankCol = Puzzle.SIZE - 1;
		tiles[blankRow][blankCol] = "";
		modelChanged();
	}

	public void pushTile(int row, int col){
		if (row == blankRow) {
			for ( ; blankCol < col; blankCol++) {
				tiles[blankRow][blankCol] = tiles[blankRow][blankCol+1];
			}
			for ( ; blankCol > col; blankCol--) {
				tiles[blankRow][blankCol] = tiles[blankRow][blankCol-1];
			}
		} else if (col == blankCol) {
			for ( ; blankRow < row; blankRow++) {
				tiles[blankRow][blankCol] = tiles[blankRow+1][blankCol];
			}
			for ( ; blankRow > row; blankRow--) {
				tiles[blankRow][blankCol] = tiles[blankRow-1][blankCol];
			}
		}
		tiles[blankRow][blankCol] = "";
		modelChanged();
	}

	private View view;
	
	public void setView(View view){
		this.view = view;
		modelChanged();
	}

	private void modelChanged(){
		if(view != null){
			// The form in which the View expects to get the state happens
			// to be the same as the form in which we store the tiles --
			// an array of arrays of Strings. However, we make a fresh
			// copy so as to ensure isolation.
			String[][] state = new String[Puzzle.SIZE][Puzzle.SIZE];
			for(int row = 0; row < Puzzle.SIZE; row++){
				for(int col = 0; col < Puzzle.SIZE; col++){
					state[row][col] = tiles[row][col];
				}
			}
			view.displayState(state);
		}
	}

}
