/**
 * Includes portions modified by Max Hailperin from files carrying this notice:
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with	the terms of the License at:
 * http://java.net/projects/javaeetutorial/pages/BerkeleyLicense
 */
package edu.gac.max.puzzle;

import java.io.StringReader;
import javax.jms.JMSException;
import javax.jms.JMSRuntimeException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.jms.Destination;
import javax.jms.JMSContext;
import javax.json.Json;
import javax.json.JsonReader;
import javax.json.JsonArray;
import javax.json.JsonArrayBuilder;

/**
 * The ProxyView class implements the MessageListener interface by defining
 * an onMessage method that triggers a Model method from a TextMessage.
 *
 * The ProxyView class also implements the View interface by
 * defining a displayState method that sends a TextMessage.
 */
public class ProxyView implements MessageListener, View {

	private String jsonState = ""; // kept around so it can be re-sent
	private Model model;
	private Destination states;
	private JMSContext context;
	
	public void onMessage(Message m) {
		try {
			TextMessage tm = (TextMessage) m;
			String s = m.getBody(String.class);
			JsonReader r = Json.createReader(new StringReader(s));
			JsonArray a = r.readArray();
			r.close();
			String command = a.getString(0);
			if(command.equals("init")){
				model.initializeTiles();
			} else if(command.equals("push")){
				model.pushTile(a.getInt(1), a.getInt(2));
			} else if(command.equals("send")){
				send();
			} else {
				System.err.println("Unrecognized command in onMessage: " + command);
			}
		} catch (Exception e) {
			System.err.println("Exception in onMessage(): " + e);
		}
	}

	private void send(){
		context.createProducer().send(states, jsonState);
	}

	public void displayState(String[][] state){
		JsonArrayBuilder outerBuilder = Json.createArrayBuilder();
		for(int row = 0; row < Puzzle.SIZE; row++){
			JsonArrayBuilder innerBuilder = Json.createArrayBuilder();
			for(int col = 0; col < Puzzle.SIZE; col++){
				innerBuilder.add(state[row][col]);
			}
			outerBuilder.add(innerBuilder);
		}
		jsonState = outerBuilder.build().toString();
		send();
	}

	public ProxyView(Model model, Destination commands, Destination states, JMSContext context){
		this.model = model;
		this.states = states;
		this.context = context;
		context.createConsumer(commands).setMessageListener(this);
	}
}
